// definition of channels that shall be pooled by the pooler
struct pooler_def {
  // enum is not necessary, ints would be enough
  enum ids {
    CURRENT_CHAN = 0,
    VCC_CHAN = 1,
    TEMP_CHAN = 2,
    POS_CHAN = 3,
  };

  /// channels that can be pooled
  multi_read_channel<CURRENT_CHAN, 128> current_chan;
  channel<VCC_CHAN> vcc_chan;
  channel<TEMP_CHAN> temp_chan;
  channel<POS_CHAN> pos_chan;

  /// pooling sequence
  static constexpr std::array sequence = {
      CURRENT_CHAN, POS_CHAN, CURRENT_CHAN, VCC_CHAN,
      CURRENT_CHAN, POS_CHAN, CURRENT_CHAN, TEMP_CHAN,
  };
};

// the pooler itself gets definition as template parameter
// it will contain pooler_def _instance_ inside
using pooler = adc_pooler<pooler_def>;

/// args are TBD
pooler p{...};

// how to setup callbacks/get data:

p->current_chan.set_callback([&](std::span<uint16_t> data) {
  // current chan is multiread, so it returns array, not that the capacity
  // specified was 128 but `data.size()` might be smaller
});

p->temp_chan.set_callback([&](uint16_t temp) {
  // we got a reading \o/
});

// or we ask for last reading
uint16_t temp = p->temp_chan.get_latest();

// how to use the pooler:
// note: the pooler internaly keeps a "state" - which channel is _currently_
// active

p.start(); // starts a reading - only one

bool finished = p.is_finished(); // has the reading finished

p.stop(); // stop the reading before ending

p.next(); // iterates the internal state to next channel based on `sequence` in
          // the definition

pooler_def::ids id =
    p.get_active_channel_id(); // you can query the active channel
